
[project]
name = "{{ name }}"
version = "0.1.0"
dependencies = []
requires-python = ">=3.8"
{%- if author %}
authors = [
    { name = "{{ author.name }}", email = "{{ author.email }}" }
]
{%- endif %}
description = "{{ description }}"
readme = "README.md"
classifiers = [
  "Programming Language :: Python"
]

{% if installable -%}
[build-system]
requires = ["hatchling"]
build-backend = "hatchling.build"

[tool.hatch.build.target.wheel]
packages = ["src/{{ safe_name }}"]
{% endif %}
